#include "../Software.h"
#include "../../../Misc.h"

#include <X11/X.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

static Window window;
static XImage* framebufferImage;
static unsigned char* framebufferPixels;
static int framebufferPitch;
static unsigned char* xfriendlyFB;
static GC gc;
static int screenWidth;
static int screenHeight;
static Pixmap intermediate;
extern Display* xDisplay;
extern Visual* xVisual;

bool WindowBackend_Software_CreateWindow(const char *window_title, size_t screen_width, size_t screen_height, bool fullscreen) {
    screenWidth = screen_width;
    screenHeight = screen_height;

    int screen = DefaultScreen(xDisplay);
    Window rootWindow = RootWindow(xDisplay, screen);

    window = XCreateWindow(xDisplay, rootWindow, 0, 0, screen_width, screen_height, 0, 24, InputOutput, xVisual, 0, NULL);
    //window = XCreateSimpleWindow(xDisplay, rootWindow, 0, 0, screen_width, screen_height, 1, 0, 0);
    XSelectInput(xDisplay, window, ExposureMask | KeyPressMask | KeyReleaseMask);
    XMapRaised(xDisplay, window);
    XStoreName(xDisplay, window, window_title);

    // Make the close button work. This is also required for the window to show up.
    // weird
    Atom WM_DELETE_WINDOW = XInternAtom(xDisplay, "WM_DELETE_WINDOW", false);
    XSetWMProtocols(xDisplay, window, &WM_DELETE_WINDOW, 1);

    gc = DefaultGC(xDisplay, DefaultScreen(xDisplay));

    int depth = DefaultDepth(xDisplay, DefaultScreen(xDisplay));

    intermediate = XCreatePixmap(xDisplay, window, screenWidth, screenHeight, depth);

    // Setting up the framebuffer
    framebufferPixels = (unsigned char*)malloc(screen_width * screen_height * 4);
    //xfriendlyFB = (unsigned char*)malloc(screen_width * screen_height * 4);
    xfriendlyFB = framebufferPixels;
    framebufferImage = XCreateImage(xDisplay, xVisual,//DefaultVisual(xDisplay, DefaultScreen(xDisplay)),
                                    depth, ZPixmap, 0,
                                    (char*)xfriendlyFB, screen_width, screen_height,
                                    32, 0);
    if(framebufferImage == 0) {
        return false;
    }
    framebufferPitch = screen_width * 4;
    return true;
}

unsigned char* WindowBackend_Software_GetFramebuffer(size_t *pitch)
{
	*pitch = framebufferPitch;

	return (unsigned char*)framebufferPixels;
}

void WindowBackend_Software_DestroyWindow(void) {
    XDestroyWindow(xDisplay, window);
}

void WindowBackend_Software_HandleWindowResize(size_t width, size_t height) {
	(void)width;
	(void)height;

    Backend_PrintError("Bro I can't resize, I'm too lazy.");
}

void WindowBackend_Software_Display(void) {
    // Convert Cave Story's framebuffer format to the Sun's
    // This is the exact reverse of how it is on modern
    // computers. Yup, it's endianess.
    // int srci = 0;
    // for(int i = 0; i < screenWidth * screenHeight * 4; i += 4) {
    //     int src;
    //     xfriendlyFB[i + 3] = 0; // UNUSED
    //     xfriendlyFB[i + 2] = framebufferPixels[srci + 0]; // BLUE
    //     xfriendlyFB[i + 1] = framebufferPixels[srci + 1]; // GREEN
    //     xfriendlyFB[i + 0] = framebufferPixels[srci + 2]; // RED
    //     srci += 4;
    // }
    //memcpy(xfriendlyFB, framebufferPixels, screenWidth * screenHeight * 4);
    XPutImage(xDisplay, window, gc, framebufferImage, 0, 0, 0, 0, screenWidth, screenHeight);
}
